--
-- Apache Derby - Off to the Races
-- Copyright (c) 2005 George Baklarz, Paul Zikopoulos, Dan Scott
--

connect 'jdbc:derby:YMLD;create=true;user=APP;password=APP';

--
-- Subquery usage
--

-- Show count by performance

SELECT 
  PRODUCTION_NO, COUNT(*) 
FROM 
  PERFORMANCES
GROUP BY PRODUCTION_NO;

-- Calculate SEAT %

SELECT 
  PRODUCTION_NO, COUNT(*),
  SUM(PERFORMANCE_SEATS) AS SEAT_COUNT,
  COUNT(*)*20 AS MAX_SEATS, 
  (SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20) AS PCNT_FULL 
FROM 
  PERFORMANCES
GROUP BY PRODUCTION_NO;

-- Restrict result set to those performances having less than 50% full

SELECT 
  PRODUCTION_NO, COUNT(*),
  SUM(PERFORMANCE_SEATS) AS SEAT_COUNT,
  COUNT(*)*20 AS MAX_SEATS, 
  (SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20) AS PCNT_FULL 
FROM 	
  PERFORMANCES
GROUP BY PRODUCTION_NO
HAVING
   ((SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20)) < 50;

-- Final Subselect example

SELECT 
  PRODUCTION_NO, PRODUCTION_TITLE
FROM
  PRODUCTIONS
WHERE
  PRODUCTION_NO IN
    (
    SELECT 
      PRODUCTION_NO,  
      (SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20) AS PCNT_FULL 
    FROM 	
      PERFORMANCES
    GROUP BY PRODUCTION_NO
    HAVING
      ((SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20)) < 50
    );

-- Subselect with a date range

SELECT 
  PRODUCTION_NO, PRODUCTION_TITLE
FROM
  PRODUCTIONS PROD
WHERE
  PRODUCTION_END < '2005-08-31' AND
    (
    SELECT 
      (SUM(PERFORMANCE_SEATS)* 100) / (COUNT(*) * 20) AS PCNT_FULL 
    FROM 	
      PERFORMANCES PERF
    WHERE
      PERF.PRODUCTION_NO = PROD.PRODUCTION_NO
    ) < 50;

QUIT;