--
-- Apache Derby - Off to the Races
-- Copyright (c) 2005 George Baklarz, Paul Zikopoulos, Dan Scott
--

connect 'jdbc:derby:YMLD;create=true;user=APP;password=APP';

--
-- Triggers
--

-- Create some test tables

DROP TABLE EMPL_BONUSES; 
DROP TRIGGER Update_EMP_Bonus;

CREATE TABLE EMPL_BONUSES
  ( NAME VARCHAR(10), SALARY INT, BONUS INT );

INSERT INTO EMPL_BONUSES(NAME, SALARY)
  VALUES
    ('Jean',15000),('Ellie',18000),('Pete',20000),('Geoffrey',25000),
    ('Andrew',3000);

DROP TABLE BONUSES;

CREATE TABLE BONUSES
  (TX INT GENERATED ALWAYS AS IDENTITY, NAME VARCHAR(10), BONUS INT );

INSERT INTO BONUSES(NAME, BONUS)
  VALUES
    ('Jean',1000),('Ellie',500),('Pete',2000),('Pete',3000),
    ('Ellie',2000),('Jean',300),('Andrew',3000);

UPDATE EMPL_BONUSES
  SET BONUS = 
   (SELECT SUM(BONUSES.BONUS) FROM BONUSES
    WHERE 
      BONUSES.NAME = EMPL_BONUSES.NAME
   );

-- Create a trigger to update the employee bonus

CREATE TRIGGER Update_EMP_Bonus
  AFTER INSERT ON BONUSES
  REFERENCING NEW AS n
  FOR EACH ROW MODE DB2SQL
UPDATE EMPL_BONUSES
  SET EMPL_BONUSES.BONUS = 
    CASE
      WHEN EMPL_BONUSES.BONUS IS NULL THEN n.BONUS 
      ELSE EMPL_BONUSES.BONUS + n.BONUS
    END
  WHERE
    EMPL_BONUSES.NAME = n.NAME;

-- Before record

SELECT * FROM EMPL_BONUSES 
  WHERE NAME='Andrew';

-- Insert new Person 

INSERT INTO BONUSES(NAME,BONUS) VALUES ('Andrew',1000); 

-- Check bonus now

SELECT * FROM EMPL_BONUSES 
  WHERE NAME='Andrew';

-- Trigger to make sure the bonus is enough

CREATE TRIGGER Check_Bonus_Value
   AFTER INSERT ON BONUSES
   REFERENCING NEW AS n
   FOR EACH ROW MODE DB2SQL
   UPDATE BONUSES SET
     BONUS =
       CASE
         WHEN (n.BONUS < 500) THEN 500
         WHEN (n.BONUS > 5000) THEN 5000
         ELSE n.BONUS
       END
    WHERE 
      TX = n.TX;

-- Insert some new values and check the results

INSERT INTO EMPL_BONUSES(Name, Salary) VALUES ('Paul',28000);
INSERT INTO BONUSES(NAME,BONUS) VALUES ('Paul',-300),('Paul',6000);

SELECT * FROM EMPL_BONUSES WHERE NAME='Paul';
SELECT * FROM BONUSES WHERE NAME='Paul';

QUIT;